# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2020 Óscar García Amor <ogarcia@connectical.com>
#
# Distributed under terms of the GNU GPLv3 license.

import logging
import os
import sys

logger = logging.getLogger(__name__)

class Config:
    def __init__(self):
        self.token_cache_file = os.path.join(os.environ.get('XDG_CACHE_HOME', os.path.join(os.path.expanduser('~'), '.cache')), 'lpacli/lpacli-token')
        self.store_token = True
        self.lesspass_token = None
        self.lesspass_host = None
        self.lesspass_user = None
        self.lesspass_pass = None

    def configure(self):
        # Get host from environment
        self.lesspass_host = os.environ.get('LESSPASS_HOST')
        if self.lesspass_host == None:
            sys.exit('You must configure LESSPASS_HOST environment variable')

        # Try to access to cache to read token
        if os.path.isdir(os.path.dirname(self.token_cache_file)):
            try:
                with open(self.token_cache_file, 'r') as reader:
                    self.lesspass_token = reader.read()
            except FileNotFoundError:
                logger.info('No token file \'{}\' found on cache dir'.format(self.token_cache_file))
            except PermissionError:
                logger.error('Cannot read token file \'{}\' from cache, please check permissions'.format(self.token_cache_file))
                self.store_token = False
            # If file is empty set token to none
            self.lesspass_token = None if self.lesspass_token == '' else self.lesspass_token
        else:
            # Cache dir doesn't exists create it
            try:
                logger.debug('Creating cache dir \'{}\''.format(os.path.dirname(self.token_cache_file)))
                os.mkdir(os.path.dirname(self.token_cache_file), mode=0o700);
            except PermissionError:
                logger.error('Cannot create cache dir \'{}\' to store token, please check permissions'.format(os.path.dirname(self.token_cache_file)))
                self.store_token = False

        # If token still none try to get username and password from environment
        if self.lesspass_token == None:
            self.lesspass_user = os.environ.get('LESSPASS_USER')
            if self.lesspass_user == None:
                sys.exit('You must configure LESSPASS_USER environment variable')
            self.lesspass_pass = os.environ.get('LESSPASS_PASS')
            if self.lesspass_pass == None:
                sys.exit('You must configure LESSPASS_PASS environment variable')

config = Config()



# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2020 Óscar García Amor <ogarcia@connectical.com>
#
# Distributed under terms of the GNU GPLv3 license.

from lpacli.config import config

import logging
import requests

logger = logging.getLogger(__name__)

class LessPassApiClient:
    def __init__(self, **kwargs):
        self.lesspass_token = kwargs.get('lesspass_token', config.lesspass_token)
        self.lesspass_host = kwargs.get('lesspass_host', config.lesspass_host)
        self.lesspass_user = kwargs.get('lesspass_user', config.lesspass_user)
        self.lesspass_pass = kwargs.get('lesspass_pass', config.lesspass_pass)

    def request(self, method, uri, **kwargs):
        if self.lesspass_token is not None:
            headers = kwargs.get('headers', {})
            headers['authorization'] = 'JWT {}'.format(self.lesspass_token)
            kwargs['headers'] = headers

        url = requests.compat.urljoin(self.lesspass_host, uri)
        logger.debug('Querying {} to {}'.format(method, url))
        return requests.request(method, url, **kwargs)

    def get_token(self):
        json = {'email': self.lesspass_user, 'password': self.lesspass_pass}
        return self.request('POST', 'api/tokens/auth/', json=json)

    def update_token(self, token):
        json = {'token': token}
        return self.request('POST', 'api/tokens/refresh/', json=json)

    def get_passwords(self):
        return self.request('GET', 'api/passwords/')




# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2020 Óscar García Amor <ogarcia@connectical.com>
#
# Distributed under terms of the GNU GPLv3 license.

from lpacli.config import config
from lpacli.commands.get_pass import get_pass

import argparse
import logging

def main():
    # Set loglevel equivalents for argument parser
    log_levels = {
            0: logging.WARNING,
            1: logging.INFO,
            2: logging.DEBUG }

    # Create argument parser to get options via arguments
    parser = argparse.ArgumentParser()
    parser.add_argument('-v', '--verbose', action='count', default=0, help='be verbose (add more v to increase verbosity)')
    parser.add_argument('site', help='site to obtain password or ls to list')
    parser.set_defaults(func=get_pass)

    args = parser.parse_args()

    # Maximum loglevel is 3 if user sends more vvv we ignore it
    args.verbose = 2 if args.verbose >= 2 else args.verbose

    # Set loglevel via argument or environment (untouched warning by default)
    log_level = log_levels[args.verbose]
    logging.basicConfig(level=log_level)
    logger = logging.getLogger('lpacli')
    logger.info('Setting loglevel to {}'.format(logging.getLevelName(log_level)))

    # Configure app
    config.configure()

    # Run main app
    args.func(args)



# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2020 Óscar García Amor <ogarcia@connectical.com>
#
# Distributed under terms of the GNU GPLv3 license.

from lpacli.api.lesspassapiclient import LessPassApiClient
from lpacli.config import config

import logging
import os
import sys

logger = logging.getLogger(__name__)

class Login:
    def __init__(self, **kwargs):
        self.token_cache_file = kwargs.get('token_cache_file', config.token_cache_file)
        self.store_token = kwargs.get('store_token', config.store_token)
        self.lesspass_token = kwargs.get('lesspass_token', config.lesspass_token)
        self.lesspass_api_client = LessPassApiClient()

    def _store_token(self, token):
        try:
            with open(self.token_cache_file, 'w') as writer:
                writer.write(token)
            os.chmod(self.token_cache_file, 0o600)
        except PermissionError:
            logger.error('Cannot write token file \'{}\' from cache, please check permissions'.format(self.token_cache_file))

    def perform_login(self):
        if self.lesspass_token == None:
            response = self.lesspass_api_client.get_token()
        else:
            response = self.lesspass_api_client.update_token(self.lesspass_token)
        if response.status_code == 200:
            config.lesspass_token = response.json()['token']
            if self.store_token:
                self._store_token(config.lesspass_token)
        else:
            logger.debug('API response: {}'.format(response.text))
            if self.lesspass_token == None:
                sys.exit('Cannot perform login, API response code: {}'.format(response.status_code))
            else:
                if self.store_token:
                    self._store_token('')
                sys.exit('Your token has expired, please try login again with username and password environment variables')




# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2020 Óscar García Amor <ogarcia@connectical.com>
#
# Distributed under terms of the GNU GPLv3 license.

from lpacli.login import Login
from lpacli.api.lesspassapiclient import LessPassApiClient

from lesspass.password import generate_password

import os
import sys

def get_pass(args):
    lesspass_login = Login()
    lesspass_api_client = LessPassApiClient()

    # Perform login to API
    lesspass_login.perform_login()

    # Get passwords list
    response = lesspass_api_client.get_passwords()

    if response.status_code == 200:
        sites = response.json().get('results')
        # List sites or get site password options
        if args.site == 'ls':
            for site in sites:
                sys.stdout.write('{}\n'.format(site.get('site')))
        else:
            site_data = None
            for site in sites:
                if site.get('site') == args.site:
                    site_data = site
                    break
            if site_data == None:
                sys.stdout.write('Cannot find any data of site \'{}\'\n'.format(args.site))
            else:
                lesspass_masterpassword = os.environ.get('LESSPASS_MASTERPASS')
                site_data['digits'] = site_data.get('numbers')
                if lesspass_masterpassword == None:
                    # Print options
                    sys.stdout.write('Site: {}\n'.format(site_data.get('site')))
                    sys.stdout.write('Login: {}\n'.format(site_data.get('login')))
                    sys.stdout.write('Lowercase: {}\n'.format(site_data.get('lowercase')))
                    sys.stdout.write('Uppercase: {}\n'.format(site_data.get('uppercase')))
                    sys.stdout.write('Digits: {}\n'.format(site_data.get('digits')))
                    sys.stdout.write('Symbols: {}\n'.format(site_data.get('symbols')))
                    sys.stdout.write('Length: {}\n'.format(site_data.get('length')))
                    sys.stdout.write('Counter: {}\n'.format(site_data.get('counter')))
                else:
                    # Print password
                    sys.stdout.write('{}\n'.format(generate_password(site_data, lesspass_masterpassword)))



# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2020 Óscar García Amor <ogarcia@connectical.com>
#
# Distributed under terms of the GNU GPLv3 license.

NAME = 'lpacli'
VERSION = '0.1.1'
AUTHOR_NAME = 'Óscar García Amor'
AUTHOR_EMAIL = 'ogarcia@connectical.com'
DESCRIPTION = 'CLI helper to get LessPass paswords configuration directly from a LessPass server.'
KEYWORDS = 'lesspass api client passwords'
URL = 'https://github.com/ogarcia/lpacli'
LICENSE = 'GPLv3'
